/*
 * Copyright (C) 2009-2010 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import javax.xml.bind.JAXBContext
import javax.xml.bind.JAXBElement
import javax.xml.bind.Unmarshaller
import javax.xml.stream.XMLStreamReader
import org.apache.cxf.staxutils.StaxUtils
import org.apache.poi.hssf.usermodel.HSSFCell
import org.apache.poi.hssf.usermodel.HSSFCellStyle
import org.apache.poi.hssf.usermodel.HSSFDataFormat
import org.apache.poi.hssf.usermodel.HSSFFont
import org.apache.poi.hssf.usermodel.HSSFRow
import org.apache.poi.hssf.usermodel.HSSFSheet
import org.apache.poi.hssf.usermodel.HSSFWorkbook

class POIHelper
{
    def wsClientHelper
    HSSFFont boldFont
    Short row0height
    HSSFCellStyle boldCellStyle
    HSSFCellStyle versionCellStyle

    POIHelper(WSClientHelper h)
    {
        this.wsClientHelper = h
    }

    def setup(HSSFWorkbook wb)
    {
        this.boldFont = wb.createFont()
        this.boldFont.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD)
        this.row0height = boldFont.fontHeight * 2
        this.boldCellStyle = wb.createCellStyle()
        this.boldCellStyle.setFont(boldFont)
        HSSFDataFormat dataFormat = wb.createDataFormat()
        this.versionCellStyle = wb.createCellStyle()
        this.versionCellStyle.setDataFormat(dataFormat.getFormat('0.0'))
    }

    def readCRFFile(File inputFile)
    {
        return readXMLFile('gov.nih.nci.cadsr.domain.jaxb.Form', inputFile)
    }

    def createCrfTranslationWorkbook(crf)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'
        HSSFWorkbook wb = new HSSFWorkbook()
        setup(wb)
        createOverviewSheet(crf, wb)
        createFieldMappingSheet(crf, wb)
        createValueMappingSheet(crf, wb)
        createCdeDesignationSheet(crf, wb)
        return wb
    }

    def createCrfReportWorkbook(crf)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'
        HSSFWorkbook wb = new HSSFWorkbook()
        setup(wb)
        createReportSheet(crf, wb)
        return wb
    }

    def writeExcelFile(HSSFWorkbook wb, File outputFile)
    {
        FileOutputStream fileOut = new FileOutputStream(outputFile)
        wb.write(fileOut)
        fileOut.close()
    }

    Unmarshaller getUnmarshaller()
    {
        JAXBContext jc = JAXBContext.newInstance(
            'gov.nih.nci.cadsr.domain.jaxb' + ':' +
            'gov.nih.nci.cadsr.umlproject.domain.jaxb' + ':' +
            'gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.jaxb')
        return jc.createUnmarshaller()
    }

    def readXMLFile(String objectClassname, File inputFile)
    {
        Unmarshaller u = getUnmarshaller()
        XMLStreamReader xmlStreamReader = StaxUtils.createXMLStreamReader(
            new FileReader(inputFile))
        Class beanClass = Thread.currentThread().getContextClassLoader()
            .loadClass(objectClassname)
        JAXBElement element = u.unmarshal(xmlStreamReader, beanClass)
        def result = element.value
        assert result.class.name == objectClassname
        return result
    }

    String concatListProp(List objList, String propName, String delim)
    {
        if(!objList) return null
        String result = ''
        objList.each
        {
            if(it[propName])
            {
                if(result) result += delim
                result += it[propName]
            }
        }
        return result
    }

    Integer computeFormNumber(String s)
    {
        def formNumber = s.split(/\D+/)[0]
        if(formNumber) formNumber = Integer.valueOf(formNumber)
        return formNumber
    }

    BigDecimal computeFormRevision(String s)
    {
        def match = s =~ /(?i:Revision)\s+(\d+(\.\d*)?)/
        if(match)
        {
            assert match.count == 1, "Ambiguous form revision"
            return Double.valueOf(match[0][1])
        }
        match = s =~ /(?i:Version)\s+(\d+(\.\d*)?)/
        if(match)
        {
            assert match.count == 1, "Ambiguous form revision (version)"
            return Double.valueOf(match[0][1])
        }
        return null
    }

    boolean indicatesNormalizedCuration(String s)
    {
        if(s =~ /(?i)normalized curation/)
        {
            return true
        }
        return false
    }

    Integer computeMappingCDEPublicID(String s)
    {
        def match = s =~ /(?i:CDE)\s+(\d+)/
        if(match)
        {
            // mapping CDE is always the last one listed
            return Integer.valueOf(match[match.count - 1][1])
        }
        return null
    }

    def computeMappingCDEVersion(mappingCDEPublicID, List questionCollection)
    {
        if(!mappingCDEPublicID)
            return null
        def result = null
        questionCollection.each
        {
            question ->

            if(question.dataElement.publicID == mappingCDEPublicID)
            {
                result = question.dataElement.version
            }
        }
        return result
    }

    HSSFCell createCell(HSSFRow row, int columnIndex, int type, HSSFCellStyle style, Object value)
    {
        HSSFCell cell = row.createCell(columnIndex, type)
        if(style) cell.setCellStyle(style)
        cell.setCellValue(value)
        return cell
    }

    HSSFCell createBoldCell(HSSFRow row, int columnIndex, Object value)
    {
        return createCell(row, columnIndex, HSSFCell.CELL_TYPE_STRING, boldCellStyle, value)
    }

    HSSFCell createNormalCell(HSSFRow row, int columnIndex, Object value)
    {
        return createCell(row, columnIndex, HSSFCell.CELL_TYPE_STRING, null, value)
    }

    HSSFCell createNumericCell(HSSFRow row, int columnIndex, Object value)
    {
        return createCell(row, columnIndex, HSSFCell.CELL_TYPE_NUMERIC, null, value)
    }

    HSSFCell createVersionCell(HSSFRow row, int columnIndex, Object value)
    {
        return createCell(row, columnIndex, HSSFCell.CELL_TYPE_NUMERIC, versionCellStyle, value)
    }

    HSSFSheet createOverviewSheet(crf, HSSFWorkbook wb)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'

        HSSFRow row
        def rowNum

        def paperFormNumber = computeFormNumber(crf.longName)
        def paperFormRevision = computeFormRevision(crf.longName)

        HSSFSheet overviewSheet = wb.createSheet('Overview')

        row = overviewSheet.createRow(0)
        createBoldCell(row, 0, 'Form Long Name')
        createNormalCell(row, 1, crf.longName)

        row = overviewSheet.createRow(2)
        createBoldCell(row, 0, 'caDSR Form Public ID')
        createNormalCell(row, 1, crf.publicID)

        row = overviewSheet.createRow(3)
        createBoldCell(row, 0, 'caDSR Form Version')
        createVersionCell(row, 1, crf.version)

        row = overviewSheet.createRow(5)
        createBoldCell(row, 0, 'Paper Form Number')
        createNumericCell(row, 1, paperFormNumber)

        row = overviewSheet.createRow(6)
        createBoldCell(row, 0, 'Paper Form Revision')
        createVersionCell(row, 1, paperFormRevision)

        row = overviewSheet.createRow(9)
        createBoldCell(row, 0, 'Preferred Definition')
        createNormalCell(row, 1, crf.preferredDefinition)

        rowNum = 11
        row = overviewSheet.createRow(rowNum)
        createBoldCell(row, 0, 'Instructions')
        if(crf.instruction)
        {
            crf.instruction.item.each
            {
                createNormalCell(row, 1, it.preferredDefinition)
                rowNum++
                row = overviewSheet.createRow(rowNum)
            }
        }
        else
        {
            rowNum++
        }

        rowNum++
        row = overviewSheet.createRow(rowNum)
        createBoldCell(row, 0, 'Modules')
        crf.moduleCollection.item.each
        {
            createNormalCell(row, 1, it.longName)
            rowNum++
            row = overviewSheet.createRow(rowNum)
        }

        overviewSheet.autoSizeColumn(0.shortValue())

        return overviewSheet
    }

    HSSFSheet createFieldMappingSheet(crf, HSSFWorkbook wb)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'

        HSSFRow row
        def rowNum
        def colNum

        def paperFormNumber = computeFormNumber(crf.longName)
        def paperFormRevision = computeFormRevision(crf.longName)

        def fmsColumnHeader =
        [
            'Paper Form Number',
            'Paper Form Revision',
            'caDSR Form Public ID',
            'caDSR Form Version',
            'Module Display Order',
            'Module Public ID',
            'Module Version',
            'Module Long Name',
            'Module Instructions',
            'Module Max Repeat',
            'Normalized',
            'Mapping CDE Public ID',
            'Mapping CDE Version',
            'Question Display Order',
            'Question Long Name',
            'Question Instructions',
            'CDE Public ID',
            'CDE Version',
            'CDE Long Name',
            'CDE Preferred Name',
            'Value Domain Public ID',
            'Value Domain Version',
            'Data Type',
            'Unit of Measure',
            'Display Format',
            'Max Length',
            'Other/Local Field Name',
            'Other/Local Field Description'
        ]

        HSSFSheet fieldMappingSheet = wb.createSheet('Field Mapping')
        row = fieldMappingSheet.createRow(0)
        row.setHeight(row0height.shortValue())
        colNum = 0
        fmsColumnHeader.each
        {
            createBoldCell(row, colNum, it)
            fieldMappingSheet.autoSizeColumn(colNum.shortValue())
            colNum++
        }
        fieldMappingSheet.createFreezePane(0, 1)

        rowNum = 1
        crf.moduleCollection.item.each
        {
            module ->

            def normalizedCuration = 'No'
            def mappingCDEPublicID = null
            def mappingCDEVersion = null

            module.instruction?.item.each
            {
                if(indicatesNormalizedCuration(it.preferredDefinition))
                {
                    normalizedCuration = 'Yes'
                    mappingCDEPublicID = computeMappingCDEPublicID(it.preferredDefinition)
                    mappingCDEVersion = computeMappingCDEVersion(mappingCDEPublicID, module.questionCollection?.item)
                }
            }

            module.questionCollection.item.each
            {
                question ->

                row = fieldMappingSheet.createRow(rowNum)
                colNum = 0
                createNumericCell(row, colNum++, paperFormNumber)
                createVersionCell(row, colNum++, paperFormRevision)
                createNumericCell(row, colNum++, crf.publicID)
                createVersionCell(row, colNum++, crf.version)
                createNumericCell(row, colNum++, module.displayOrder)
                createNumericCell(row, colNum++, module.publicID)
                createVersionCell(row, colNum++, module.version)
                createNormalCell(row, colNum++, module.longName)
                createNormalCell(row, colNum++, concatListProp(module.instruction?.item, 'preferredDefinition', ' '))
                createNumericCell(row, colNum++, module.maximumQuestionRepeat ? module.maximumQuestionRepeat : 0)
                if(normalizedCuration)
                {
                    createNormalCell(row, colNum++, normalizedCuration)
                    createNumericCell(row, colNum++, mappingCDEPublicID)
                    createVersionCell(row, colNum++, mappingCDEVersion)
                }
                else
                {
                    colNum += 3
                }
                createNumericCell(row, colNum++, question.displayOrder)
                createNormalCell(row, colNum++, question.longName)
                createNormalCell(row, colNum++, concatListProp(question.instruction?.item, 'preferredDefinition', ' '))
                createNumericCell(row, colNum++, question.dataElement.publicID)
                createVersionCell(row, colNum++, question.dataElement.version)
                createNormalCell(row, colNum++, question.dataElement.longName)
                createNormalCell(row, colNum++, question.dataElement.preferredName)
                createNumericCell(row, colNum++, question.dataElement.valueDomain.publicID)
                createVersionCell(row, colNum++, question.dataElement.valueDomain.version)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.datatypeName)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.UOMName)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.formatName)
                createNumericCell(row, colNum++, question.dataElement.valueDomain.maximumLengthNumber)

                rowNum++
            }
        }

        return fieldMappingSheet
    }

    HSSFSheet createValueMappingSheet(crf, HSSFWorkbook wb)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'

        HSSFRow row
        def rowNum
        def colNum

        def paperFormNumber = computeFormNumber(crf.longName)
        def paperFormRevision = computeFormRevision(crf.longName)

        def vmsColumnHeader =
        [
            'Paper Form Number',
            'Paper Form Revision',
            'caDSR Form Public ID',
            'caDSR Form Version',
            'Module Display Order',
            'Module Public ID',
            'Module Version',
            'Module Long Name',
            'CDE Display Order',
            'CDE Public ID',
            'CDE Version',
            'CDE Long Name',
            'CDE Preferred Name',
            'Value Domain Public ID',
            'Value Domain Version',
            'Valid Value Display Order',
            'Valid Value Long Name',
            'Value Meaning Text',
            'Value Meaning Public ID',
            'Value Meaning Version',
            'Value Meaning Desc.',
            'Valid Value Instructions',
            'Other/Local Value',
            'Other/Local Value Meaning Description'
        ]

        HSSFSheet valueMappingSheet = wb.createSheet('Value Mapping')
        row = valueMappingSheet.createRow(0)
        row.setHeight(row0height.shortValue())
        colNum = 0
        vmsColumnHeader.each
        {
            createBoldCell(row, colNum, it)
            valueMappingSheet.autoSizeColumn(colNum.shortValue())
            colNum++
        }
        valueMappingSheet.createFreezePane(0, 1)
        rowNum = 1
        crf.moduleCollection.item.each
        {
            module ->

            module.questionCollection.item.each
            {
                question ->

                if(question.validValueCollection)
                {
                    question.validValueCollection.item.each
                    {
                        validValue ->

                        row = valueMappingSheet.createRow(rowNum)
                        colNum = 0
                        createNumericCell(row, colNum++, paperFormNumber)
                        createVersionCell(row, colNum++, paperFormRevision)
                        createNumericCell(row, colNum++, crf.publicID)
                        createVersionCell(row, colNum++, crf.version)
                        createNumericCell(row, colNum++, module.displayOrder)
                        createNumericCell(row, colNum++, module.publicID)
                        createVersionCell(row, colNum++, module.version)
                        createNormalCell(row, colNum++, module.longName)
                        createNumericCell(row, colNum++, question.displayOrder)
                        createNumericCell(row, colNum++, question.dataElement.publicID)
                        createVersionCell(row, colNum++, question.dataElement.version)
                        createNormalCell(row, colNum++, question.dataElement.longName)
                        createNormalCell(row, colNum++, question.dataElement.preferredName)
                        createNumericCell(row, colNum++, question.dataElement.valueDomain.publicID)
                        createVersionCell(row, colNum++, question.dataElement.valueDomain.version)
                        createNumericCell(row, colNum++, validValue.displayOrder)
                        createNormalCell(row, colNum++, validValue.longName)
                        createNumericCell(row, colNum++, validValue.valueDomainPermissibleValue.permissibleValue.valueMeaning.longName)
                        createNumericCell(row, colNum++, validValue.valueDomainPermissibleValue.permissibleValue.valueMeaning.publicID)
                        createVersionCell(row, colNum++, validValue.valueDomainPermissibleValue.permissibleValue.valueMeaning.version)
                        createNumericCell(row, colNum++, validValue.valueDomainPermissibleValue.permissibleValue.valueMeaning.preferredDefinition)
                        createNumericCell(row, colNum++, concatListProp(validValue.instruction?.item, 'preferredDefinition', ' '))

                        rowNum++
                    }
                }
            }
        }

        return valueMappingSheet
    }

    HSSFSheet createCdeDesignationSheet(crf, HSSFWorkbook wb)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'

        HSSFRow row
        def rowNum
        def colNum

        def columnHeader =
        [
            'CDE Public ID',
            'CDE Version',
            'CDE Long Name',
            'CDE Preferred Name',
            'Designation Type',
            'Designation Name'
        ]

        // extract set of distinct CDEs
        def cdeMap = [:]
        crf.moduleCollection.item.each
        {
            module ->

            module.questionCollection.item.each
            {
                question ->

                cdeMap[question.dataElement.id] = question.dataElement
            }
        }

        // sort by (publicID, version)
        def idList = cdeMap.keySet().toList()
        idList.sort
        {
            a, b ->

            def result = cdeMap[a].publicID <=> cdeMap[b].publicID
            if(result == 0)
            {
                result = cdeMap[a].version <=> cdeMap[b].version
            }
            return result
        }

        HSSFSheet sheet = wb.createSheet('CDE Designation')
        row = sheet.createRow(0)
        row.setHeight(row0height.shortValue())
        colNum = 0
        columnHeader.each
        {
            createBoldCell(row, colNum, it)
            sheet.autoSizeColumn(colNum.shortValue())
            colNum++
        }
        sheet.createFreezePane(0, 1)
        rowNum = 1
        idList.each
        {
            id ->

            def dataElement = cdeMap[id]
            if(dataElement.designationCollection)
            {
                dataElement.designationCollection.item.each
                {
                    designation ->

                    row = sheet.createRow(rowNum)
                    colNum = 0
                    createNumericCell(row, colNum++, dataElement.publicID)
                    createVersionCell(row, colNum++, dataElement.version)
                    createNormalCell(row, colNum++, dataElement.longName)
                    createNormalCell(row, colNum++, dataElement.preferredName)
                    createNormalCell(row, colNum++, designation.type)
                    createNormalCell(row, colNum++, designation.name)

                    rowNum++
                }
            }
        }

        return sheet
    }

    HSSFSheet createReportSheet(crf, HSSFWorkbook wb)
    {
        assert crf.class.name == 'gov.nih.nci.cadsr.domain.jaxb.Form'

        HSSFRow row
        def rowNum
        def colNum

        HSSFSheet sheet = wb.createSheet('Sheet0')

        def columnHeader =
        [
            'Module Display Order',
            'Question Display Order',
            'Valid Value Display Order',
            'Module Long Name',
            'Module Instructions',
            'Module Public ID',
            'Module Version',
            'Number of Repetitions',
            'Normalized Curation',
            'Mapping CDE Public ID',
            'Mapping CDE Version',
            'Question Long Name',
            'Question Instructions',
            'CDE Long Name',
            'CDE Preferred Name',
            'CDE Public ID',
            'CDE Version',
            'Answer is Mandatory',
            'Question Default Value',
            'Value Domain Long Name',
            'Data Type',
            'Unit of Measure',
            'Display Format',
            'Max Length',
            'Concepts',
            'Valid Value',
            'Value Meaning Text',
            'Value Meaning Public ID',
            'Value Meaning Version',
            'Value Meaning Desc.',
            'Valid Value Instructions',
        ]

        row = sheet.createRow(10)
        colNum = 0
        columnHeader.each
        {
            createBoldCell(row, colNum, it)
            sheet.autoSizeColumn(colNum.shortValue())
            colNum++
        }
        sheet.createFreezePane(0, 11)

        row = sheet.createRow(0)
        createBoldCell(row, 0, 'Long Name')
        createNormalCell(row, 1, crf.longName)

        row = sheet.createRow(1)
        createBoldCell(row, 0, 'Definition')
        createNormalCell(row, 1, crf.preferredDefinition)

        row = sheet.createRow(2)
        createBoldCell(row, 0, 'Context')
//        createNormalCell(row, 1, crf.context.longName)

        row = sheet.createRow(3)
        createBoldCell(row, 0, 'Protocol Long Name')
        createNormalCell(row, 1, concatListProp(crf.protocolCollection?.item, 'longName', ', '))

        row = sheet.createRow(4)
        createBoldCell(row, 0, 'Workflow')
        createNormalCell(row, 1, crf.workflowStatusName)

        row = sheet.createRow(5)
        createBoldCell(row, 0, 'Type')
        createNormalCell(row, 1, crf.type)

        row = sheet.createRow(6)
        createBoldCell(row, 0, 'Public ID')
        createNumericCell(row, 1, crf.publicID)

        row = sheet.createRow(7)
        createBoldCell(row, 0, 'Version')
        createVersionCell(row, 1, crf.version)

        row = sheet.createRow(8)
        createBoldCell(row, 0, 'Header Instruction')
        createNormalCell(row, 1, concatListProp(crf.instruction?.item, 'preferredDefinition', ' '))

        rowNum = 11
        crf.moduleCollection.item.each
        {
            module ->

            def normalizedCuration = 'No'
            def mappingCDEPublicID = null
            def mappingCDEVersion = null

            module.instruction?.item.each
            {
                if(indicatesNormalizedCuration(it.preferredDefinition))
                {
                    normalizedCuration = 'Yes'
                    mappingCDEPublicID = computeMappingCDEPublicID(it.preferredDefinition)
                    mappingCDEVersion = computeMappingCDEVersion(mappingCDEPublicID, module.questionCollection?.item)
                }
            }

            row = sheet.createRow(rowNum)
            colNum = 0
            createNumericCell(row, colNum++, module.displayOrder)
            colNum += 2
            createNormalCell(row, colNum++, module.longName)
            createNormalCell(row, colNum++, concatListProp(module.instruction?.item, 'preferredDefinition', ' '))
            createNumericCell(row, colNum++, module.publicID)
            createVersionCell(row, colNum++, module.version)
            createNumericCell(row, colNum++, module.maximumQuestionRepeat ? module.maximumQuestionRepeat : 0)
            createNormalCell(row, colNum++, normalizedCuration)
            createNumericCell(row, colNum++, mappingCDEPublicID)
            createVersionCell(row, colNum++, mappingCDEVersion)
            rowNum++

            module.questionCollection.item.each
            {
                question ->

                row = sheet.createRow(rowNum)
                colNum = 0
                createNumericCell(row, colNum++, module.displayOrder)
                createNumericCell(row, colNum++, question.displayOrder)
                colNum += 9
                createNormalCell(row, colNum++, question.longName)
                createNormalCell(row, colNum++, concatListProp(question.instruction?.item, 'preferredDefinition', ' '))
                createNormalCell(row, colNum++, question.dataElement.longName)
                createNormalCell(row, colNum++, question.dataElement.preferredName)
                createNumericCell(row, colNum++, question.dataElement.publicID)
                createVersionCell(row, colNum++, question.dataElement.version)
                createNormalCell(row, colNum++, question.isMandatory ? question.isMandatory : 'No')
                createNormalCell(row, colNum++, question.defaultValue)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.longName)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.datatypeName)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.UOMName)
                createNormalCell(row, colNum++, question.dataElement.valueDomain.formatName)
                createNumericCell(row, colNum++, question.dataElement.valueDomain.maximumLengthNumber)
                rowNum++

                question.validValueCollection?.item.each
                {
                    validValue ->

                    row = sheet.createRow(rowNum)
                    colNum = 0
                    createNumericCell(row, colNum++, module.displayOrder)
                    createNumericCell(row, colNum++, question.displayOrder)
                    createNumericCell(row, colNum++, validValue.displayOrder)
                    colNum += 22
                    createNormalCell(row, colNum++, validValue.longName)
                    def valueMeaning = validValue.valueDomainPermissibleValue?.permissibleValue?.valueMeaning
                    createNormalCell(row, colNum++, valueMeaning?.longName)
                    createNumericCell(row, colNum++, valueMeaning?.publicID)
                    createVersionCell(row, colNum++, valueMeaning?.version)
                    createNormalCell(row, colNum++, valueMeaning?.preferredDefinition)
                    createNormalCell(row, colNum++, concatListProp(validValue.instruction?.item, 'preferredDefinition', ' '))
                    rowNum++
                }
            }
        }

        return sheet
    }
}