/*
 * Copyright (C) 2009 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import org.apache.commons.logging.Log
import org.apache.commons.logging.LogFactory

abstract class Retriever
{
    static Log log = LogFactory.getLog(Retriever.class)
    Boolean oneToMany = false
    List propertyNameList = []
    List orderBy = [ 'id' ]
    RetrieverFactory factory = null

    abstract Object retrieve(WSClient wsClient, Object parentObj)

    def addPropertyName(propertyName)
    {
        this.propertyNameList.add(propertyName)
        return this
    }

    def int compare(a, b)
    {
        def result = 0
        if(this.orderBy != null)
        {
            this.orderBy.each
            {
                if(result == 0)
                {
                    result = a[it] <=> b[it]
                }
            }
        }
        return result
    }

    def computeRetrievalResult(WSClient wsClient, List wsResultList)
    {
        if(!wsResultList)
        {
            return null
        }

        // sort the list
        def sortedResultList = wsResultList.sort { a, b -> compare(a, b) }

        // retrieve parts of each list entry
        sortedResultList.each
        {
            obj ->

            this.propertyNameList?.each
            {
                propertyName ->

                retrievePart(obj, propertyName, wsClient)
            }

            if(this.factory.progressMonitor)
            {
                if(this.factory.progressClassName == obj.class.name)
                {
                    this.factory.progressAmount++
                }
                this.factory.progressMonitor.progress = this.factory.progressAmount
            }
        }

        if(this.factory.progressMonitor?.isCanceled())
        {
            return null
        }

        // return single object if indicated
        if(!this.oneToMany)
        {
            assert sortedResultList.size == 1
            return sortedResultList[0]
        }

        // return ArrayOfXsdAnyType constructed from list
        def result = wsClient.create(
            'gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.' +
            'jaxb.ArrayOfXsdAnyType')
        result.item = sortedResultList
        return result
    }

    void retrievePart(Object bean, String propertyName, WSClient wsClient)
    {
        log.trace('retrievePart: ' + bean + ', ' + propertyName + ', WSClient')

        if(this.factory.progressMonitor?.isCanceled())
        {
            return null
        }

        def classProp = propertyName.split('::')
        if(classProp.length == 1)
        {
            // retrieve property from web service
            bean[propertyName] = factory.newRetriever(propertyName).retrieve(wsClient, bean)
        }
        else
        {
            // only retrieve property for obj of specified class
            // (e.g. EnumeratedValueDomain)
            def className = classProp[0]
            def propName = classProp[1]
            log.trace('className: ' + className + ' ' + bean.getClass().name)
            if(bean.getClass().name.equals(className))
            {
                log.trace('propName: ' + propName)
                // retrieve property from web service
                bean[propName] = factory.newRetriever(propName).retrieve(wsClient, bean)
            }
        }
    }
}