/*
 * Copyright (C) 2009-2010 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import javax.swing.ProgressMonitor

class RetrieverFactory
{
    def progressMonitor
    def progressClassName
    def progressAmount
    Boolean expandValueDomain = false

    RetrieverFactory()
    {
        this(null, null)
    }

    RetrieverFactory(ProgressMonitor progressMonitor, String progressClassName)
    {
        this.progressMonitor = progressMonitor
        this.progressClassName = progressClassName
        if(progressMonitor) this.progressAmount = (int)(progressMonitor.maximum / 100) + 1
    }

    RetrieverFactory(ProgressMonitor progressMonitor, String progressClassName, Boolean expandValueDomain)
    {
        this(progressMonitor, progressClassName)
        this.expandValueDomain = expandValueDomain
    }

    def newRetriever(propertyName)
    {
        def instance = null;

        switch(propertyName)
        {
            case 'administeredComponent' :
                instance = new Target('AdministeredComponent')
                instance.orderBy = [ 'publicID', 'version' ]
                //instance.addPropertyName('administeredComponentClassSchemeItemCollection')
                //instance.addPropertyName('administeredComponentContactCollection')
                //instance.addPropertyName('context')
                instance.addPropertyName('definitionCollection')
                instance.addPropertyName('designationCollection')
                instance.addPropertyName('referenceDocumentCollection')
                //instance.addPropertyName('registrations')
                break;

            case 'administeredComponentClassSchemeItemCollection' :
                instance = new Association(propertyName)
                instance.oneToMany = true
                instance.addPropertyName('classSchemeClassSchemeItem')
                //instance.addPropertyName('sourceObjectClass')
                //instance.addPropertyName('targetObjectClass')
                break;

            case 'administeredComponentCollection' :
                instance = newRetriever('administeredComponent')
                instance.oneToMany = true
                break;

            case 'attachments' :
                instance = new Target('ReferenceDocumentAttachment')
                instance.oneToMany = true;
                ////instance.addPropertyName('referenceDocument')
                break;

            case 'classificationScheme' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ClassificationScheme'
                //instance.addPropertyName('childClassificationScheme')
                //instance.addPropertyName('childClassificationSchemeRelationshipCollection')
                //instance.addPropertyName('classSchemeClassSchemeItemCollection')
                //instance.addPropertyName('conceptDerivationRule')
                //instance.addPropertyName('parentClassificationSchemeRelationshipCollection')
                //instance.addPropertyName('projectCollection')
                break;

            case 'classificationSchemeItem' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ClassificationSchemeItem'
                break;

            case 'classSchemeClassSchemeItem' :
                instance = new Target('ClassSchemeClassSchemeItem')
                instance.orderBy = [ 'displayOrder', 'id' ]
                //instance.addPropertyName('UMLPackageMetadataCollection')
                ////instance.addPropertyName('administeredComponentClassSchemeItemCollection')
                //instance.addPropertyName('administeredComponentContact')
                //instance.addPropertyName('association')
                //instance.addPropertyName('childClassSchemeClassSchemeItemCollection')
                instance.addPropertyName('classificationScheme')
                //instance.addPropertyName('classificationSchemeItem')
                //instance.addPropertyName('definitionClassSchemeItemCollection')
                //instance.addPropertyName('designationClassSchemeItemCollection')
                //instance.addPropertyName('referenceDocumentCollection')
                //instance.addPropertyName('subProjectCollection')
                break;

            case 'componentConcept' :
                instance = new Target('ComponentConcept')
                instance.addPropertyName('componentlevel')
                instance.addPropertyName('concept')
                ////instance.addPropertyName('derivationRule')
                break;

            case 'componentConceptCollection' :
                instance = newRetriever('componentConcept')
                instance.oneToMany = true
                break;

            case 'componentlevel' :
                instance = new Target('ComponentLevel')
                break;

            case 'concept' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'Concept'
                ////instance.addPropertyName('componentConceptCollection')
                instance.addPropertyName('semanticMetadata')
                ////instance.addPropertyName('valueDomainPermissibleValueCollection')
                break;

            case 'conceptDerivationRule' :
                instance = new Target('ConceptDerivationRule')
                ////instance.addPropertyName('classificationSchemeCollection')
                ////instance.addPropertyName('classificationSchemeItemCollection')
                instance.addPropertyName('componentConceptCollection')
                ////instance.addPropertyName('conceptualDomainCollection')
                break;

            case 'conceptualDomain' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ConceptualDomain'
                instance.addPropertyName('conceptDerivationRule')
                ////instance.addPropertyName('dataElementConceptCollection')
                ////instance.addPropertyName('valueDomainCollection')
                //instance.addPropertyName('valueMeaningCollection')
                break;

            case 'dataElement' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'DataElement'
                //instance.addPropertyName('attributeMetadata')
                //instance.addPropertyName('childDataElementRelationshipsCollection')
                instance.addPropertyName('dataElementConcept')
                //instance.addPropertyName('dataElementDerivationCollection')
                //instance.addPropertyName('derivedDataElement')
                //instance.addPropertyName('parentDataElementRelationshipsCollection')
                ////instance.addPropertyName('questionCollection')
                instance.addPropertyName('valueDomain')
                break;

            case 'dataElementConcept' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'DataElementConcept'
                //instance.addPropertyName('childDataElementConceptRelationshipCollection')
                //instance.addPropertyName('conceptualDomain')
                ////instance.addPropertyName('dataElementCollection')
                //instance.addPropertyName('objectClass')
                //instance.addPropertyName('parentDataElementConceptRelationshipCollection')
                //instance.addPropertyName('property')
                break;

            case 'dataElementDerivation' :
                instance = new Target('DataElementDerivation')
                ////instance.addPropertyName('dataElement')
                ////instance.addPropertyName('derivedDataElement')
                //instance.addPropertyName('leftOperand')
                break;

            case 'dataElementDerivationCollection' :
                instance = newRetriever('dataElementDerivation')
                instance.oneToMany = true
                break;

            case 'definitionCollection' :
                instance = new Target('Definition')
                instance.oneToMany = true
                instance.orderBy = [ 'type', 'id' ]
                //instance.addPropertyName('context')
                //instance.addPropertyName('definitionClassSchemeItemCollection')
                break;

            case 'designationCollection' :
                instance = new Target('Designation')
                instance.oneToMany = true
                instance.orderBy = [ 'type', 'name', 'id' ]
                //instance.addPropertyName('context')
                //instance.addPropertyName('designationClassSchemeItemCollection')
                break;

            case 'form' :
                instance = newRetriever('formElement')
                instance.targetClassName = 'Form'
                instance.addPropertyName('moduleCollection')
                instance.addPropertyName('protocolCollection')
                break;

            case 'formCollection' :
                instance = newRetriever('form')
                instance.oneToMany = true
                break;

            case 'formElement' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'FormElement'
                instance.addPropertyName('instruction')
                //instance.addPropertyName('triggerActionCollection')
                break;

            case 'instruction' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'Instruction'
                instance.oneToMany = true
                instance.orderBy = [ 'type', 'publicID', 'version' ]
                ////instance.addPropertyName('formElement')
                break;

            case 'module' :
                instance = newRetriever('formElement')
                instance.targetClassName = 'Module'
                instance.orderBy = [ 'displayOrder', 'publicID', 'version' ]
                ////instance.addPropertyName('form')
                instance.addPropertyName('questionCollection')
                break;

            case 'moduleCollection' :
                instance = newRetriever('module')
                instance.oneToMany = true
                break;

            case 'objectClass' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ObjectClass'
                //instance.addPropertyName('UMLClassMetadata')
                instance.addPropertyName('conceptDerivationRule')
                ////instance.addPropertyName('dataElementConceptCollection')
                //instance.addPropertyName('sourcObjectClassRelationshipCollection')
                //instance.addPropertyName('targetObjectClassRelationshipCollection')
                break;

            case 'permissibleValue' :
                instance = new Target('PermissibleValue')
                ////instance.addPropertyName('valueDomainPermissibleValueCollection')
                instance.addPropertyName('valueMeaning')
                break;

            case 'property' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'Property'
                instance.addPropertyName('conceptDerivationRule')
                ////instance.addPropertyName('dataElementConceptCollection')
                break;

            case 'protocol' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'Protocol'
                ////instance.addPropertyName('formCollection')
                break;

            case 'protocolCollection' :
                instance = newRetriever('protocol')
                instance.oneToMany = true
                break;

            case 'question' :
                instance = newRetriever('formElement')
                instance.targetClassName = 'Question'
                instance.orderBy = [ 'displayOrder', 'publicID', 'version' ]
                instance.addPropertyName('dataElement')
                ////instance.addPropertyName('module')
                //instance.addPropertyName('questionComponentCollection')
                //instance.addPropertyName('questionCondition')
                //instance.addPropertyName('questionRepetitionCollection')
                instance.addPropertyName('validValueCollection')
                //instance.addPropertyName('valueDomain')
                break;

            case 'questionCollection' :
                instance = newRetriever('question')
                instance.oneToMany = true
                break

            case 'referenceDocumentCollection' :
                instance = new Association(propertyName)
                instance.oneToMany = true
                instance.orderBy = [ 'displayOrder', 'type', 'name', 'id' ]
                instance.addPropertyName('attachments')
                //instance.addPropertyName('classSchemeClassSchemeItem')
                //instance.addPropertyName('context')
                //instance.addPropertyName('registration')
                break;

            case 'represention' :
                instance = new Target('AdministeredComponent')
                instance.targetClassName = 'Representation'
                instance.addPropertyName('conceptDerivationRule')
                ////instance.addPropertyName('valueDomainCollection')
                break

            case 'semanticMetadata' :
                instance = new Association('semanticMetadata')
                instance.oneToMany = true
                //instance.addPropertyName('UMLAssociationMetadata')
                //instance.addPropertyName('UMLAttributeMetadata')
                //instance.addPropertyName('UMLClassMetadata')
                //instance.addPropertyName('attributeTypeMetadata')
                ////instance.addPropertyName('concept')
                //instance.addPropertyName('typeEnumerationMetadata')
                break;

            case 'validValueCollection' :
                instance = newRetriever('formElement')
                instance.targetClassName = 'ValidValue'
                instance.oneToMany = true
                instance.orderBy = [ 'displayOrder', 'publicID', 'version' ]
                //instance.addPropertyName('conditionComponent')
                ////instance.addPropertyName('question')
                instance.addPropertyName('valueDomainPermissibleValue')
                break;

            case 'valueDomain' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ValueDomain'
                //instance.addPropertyName('attributeTypeMetadata')
                //instance.addPropertyName('childValueDomainRelationshipCollection')
                //instance.addPropertyName('conceptDerivationRule')
                //instance.addPropertyName('conceptualDomain')
                ////instance.addPropertyName('dataElementCollection')
                //instance.addPropertyName('parentValueDomainRelationshipCollection')
                ////instance.addPropertyName('questionCollection')
                //instance.addPropertyName('represention')
                if(expandValueDomain)
                {
                    instance.addPropertyName('gov.nih.nci.cadsr.domain.jaxb.EnumeratedValueDomain::valueDomainPermissibleValueCollection')
                }
                break;

            case 'valueDomainPermissibleValue' :
                instance = new Target('ValueDomainPermissibleValue')
                //instance.addPropertyName('concept')
                ////instance.addPropertyName('enumeratedValueDomain')
                instance.addPropertyName('permissibleValue')
                //instance.addPropertyName('validValueCollection')
                break;

            case 'valueDomainPermissibleValueCollection' :
                instance = newRetriever('valueDomainPermissibleValue')
                instance.oneToMany = true
                break;

            case 'valueMeaning' :
                instance = newRetriever('administeredComponent')
                instance.targetClassName = 'ValueMeaning'
                //instance.addPropertyName('conceptDerivationRule')
                //instance.addPropertyName('conceptualDomainCollection')
                ////instance.addPropertyName('permissibleValueCollection')
                break;

            default:
                throw new IllegalArgumentException(
                    'Unrecognized property name: ' + propertyName)
        }

        assert instance != null
        instance.factory = this

        return instance
    }
}