/*
 * Copyright (C) 2009 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import gov.nih.nci.system.client.ApplicationServiceProvider;
import gov.nih.nci.system.query.hibernate.HQLCriteria
import org.apache.commons.logging.Log
import org.apache.commons.logging.LogFactory

class Target
extends Retriever
{
    static Log log = LogFactory.getLog(Target.class) 
    String targetPackageName = 'gov.nih.nci.cadsr.domain'
    String targetClassName

    Target(targetClassName)
    {
        this.targetClassName = targetClassName
    }

    def qualifiedTargetName()
    {
        assert targetPackageName
        assert targetClassName
        return targetPackageName + '.' + targetClassName
    }

    def hqlRetrieveAccsiDataElement(WSClient wsClient, Object parentObj)
    {
        // perform HQL query to retrieve DataElement for ACCSI

        // (appService is configured via application-config-client.xml)
        def appService = ApplicationServiceProvider.getApplicationService()
		def hql =
			'from gov.nih.nci.cadsr.domain.DataElement de' +
			' where de.administeredComponentClassSchemeItemCollection.id' +
			' = ?'
		def resultList = appService.query(
		    new HQLCriteria(hql, [ parentObj.id ]));
        if((resultList == null) || (resultList.size() < 1))
        {
            // ACCSI is not associated with a DataElement
            return null
        }
        assert resultList.size() == 1

        // now retrieve DataElement from web service
		def criteria = wsClient.create(
		    'gov.nih.nci.cadsr.domain.jaxb.DataElement')
		criteria.id = resultList[0].id
		return wsRetrieve(wsClient, criteria)
    }

    def wsRetrieve(WSClient wsClient, Object parentObj)
    {
        def criteria = wsClient.create(parentObj.class.name)
        criteria.id = parentObj.id
        def wsResult
        synchronized(wsClient)
        {
            wsResult = wsClient.queryObject(qualifiedTargetName(), criteria)
        }
        return computeRetrievalResult(wsClient, wsResult.queryObjectReturn)
    }

    def retrieve(WSClient wsClient, Object parentObj)
    {
        log.trace('retrieve: ' + parentObj.class.name + ': ' +
            targetClassName + System.properties['line.separator'] +
            parentObj.id)
        if((targetClassName == 'DataElement') && (parentObj.class.name ==
            'gov.nih.nci.cadsr.domain.jaxb.AdministeredComponentClassSchemeItem'))
        {
            // special handling for ACCSI DataElement
            return hqlRetrieveAccsiDataElement(wsClient, parentObj)
        }
        else
        {
            return wsRetrieve(wsClient, parentObj)
        }
   }
}