/*
 * Copyright (C) 2009 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.jaxb.ArrayOfXsdAnyType
import gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.WSQueryImpl
import gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.WSQueryImplService
import javax.xml.namespace.QName
import net.agnis.cadsrapi41.cxf.PostStaxInInterceptor
import org.apache.commons.logging.Log
import org.apache.commons.logging.LogFactory
import org.apache.cxf.configuration.security.AuthorizationPolicy
import org.apache.cxf.endpoint.Client
import org.apache.cxf.frontend.ClientProxy
import org.apache.cxf.transport.http.HTTPConduit
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy

/**
 * Replacement for groovyx.net.ws.WSClient.
 * 
 * This client uses statically generated Java beans, etc., instead of the
 * dynamically generated CXF client used by GroovyWS, to reduce program
 * startup delay and eliminate potential class loader issues.
 */
class WSClient
{
    static Log log = LogFactory.getLog(WSClient.class)

    WSQueryImpl port = null

    WSClient(
        URL wsdlLocation = WSQueryImplService.WSDL_LOCATION,
        boolean useProxyServer = false,
        String proxyHost = null,
        String proxyPort = null,
        String proxyUser = null,
        String proxyPassword = null,
        boolean useHttpBasicAuthentication = false,
        String httpUser = null,
        String httpPassword = null)
    {
        assert wsdlLocation != null
        WSQueryImplService ss = new WSQueryImplService(wsdlLocation);
        port = ss.getCadsrapi41Service();

        Client client = ClientProxy.getClient(port)
        client.inInterceptors.add(new PostStaxInInterceptor())

        HTTPConduit conduit = (HTTPConduit) client.getConduit()
        HTTPClientPolicy httpClientPolicy = new HTTPClientPolicy()
        httpClientPolicy.setConnectionTimeout(0)
        httpClientPolicy.setAllowChunking(false)
        conduit.setClient(httpClientPolicy)

        if(useHttpBasicAuthentication)
        {
            AuthorizationPolicy authPolicy = conduit.getAuthorization()
            authPolicy.setUserName(httpUser)
            authPolicy.setPassword(httpPassword)
        }

        if(useProxyServer)
        {
            conduit.getClient().setProxyServer(proxyHost)
            int port = 0
            try
            {
                port = Integer.parseInt(proxyPort)
            }
            catch(NumberFormatException e)
            {
                log.error('Proxy port number: ' + proxyPort, e)
            }
            conduit.getClient().setProxyServerPort(port)
            conduit.getProxyAuthorization().setUserName(proxyUser)
            conduit.getProxyAuthorization().setPassword(proxyPassword)
        }
    }

    public void initialize()
    {
    }

    public Object create(String className)
    {
        return Thread.currentThread().getContextClassLoader().loadClass(className).newInstance()
    }

    // web service (port) methods ...

    public ArrayOfXsdAnyType query(
        String targetClassName,
        Object criteria,
        int startIndex)
    {
        return port.query(targetClassName, criteria, startIndex)
    }

    public int getTotalNumberOfRecords(
        String targetClassName,
        Object criteria)
    {
        return port.getTotalNumberOfRecords(targetClassName, criteria)
    }

    public int getRecordsPerQuery()
    {
        return port.getRecordsPerQuery()
    }

    public ArrayOfXsdAnyType getAssociation(
        Object source,
        String associationName,
        int startIndex)
    {
        return port.getAssociation(source, associationName, startIndex)
    }

    public ArrayOfXsdAnyType queryObject(
        String targetClassName,
        Object criteria)
    {
        return port.queryObject(targetClassName, criteria)
    }

    public int getMaximumRecordsPerQuery()
    {
        return port.getMaximumRecordsPerQuery()
    }

    public java.lang.String getVersion()
    {
        return port.getVersion()
    }
}