/*
 * Copyright (C) 2009-2010 National Marrow Donor Program. All Rights Reserved.
 */

package net.agnis.cadsrapi41

import javax.swing.ProgressMonitor
import javax.xml.bind.JAXBContext
import javax.xml.bind.JAXBElement
import javax.xml.bind.Marshaller
import javax.xml.namespace.QName
import net.agnis.cadsrapi41.cxf.PostStaxInInterceptor
import net.agnis.cadsrapi41.jaxb.NamespacePrefixMapper

class WSClientHelper
{
    WSClient wsClient = null

    WSClientHelper(
        URL wsdlLocation,
        boolean useProxyServer,
        String proxyHost,
        String proxyPort,
        String proxyUser,
        String proxyPassword,
        boolean useHttpBasicAuthentication,
        String httpUser,
        String httpPassword)
    {
        wsClient = new WSClient(
            wsdlLocation,
            useProxyServer,
            proxyHost,
            proxyPort,
            proxyUser,
            proxyPassword,
            useHttpBasicAuthentication,
            httpUser,
            httpPassword)
    }

    def retrieveCdeReviewClassificationScheme()
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.ClassificationScheme')
        criteria.longName = 'NMDP: CDEs to review'
        criteria.context = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Context')
        criteria.context.name = 'NHLBI'
        def wsResult
        synchronized(wsClient)
        {
            wsResult = wsClient.queryObject('gov.nih.nci.cadsr.domain.ClassificationScheme', criteria)
        }
        def cdeReviewClassificationScheme = wsResult.queryObjectReturn[0]
        assert cdeReviewClassificationScheme != null
        assert cdeReviewClassificationScheme.id != null

        criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.ClassificationScheme')
        criteria.id = cdeReviewClassificationScheme.id
        cdeReviewClassificationScheme = new ClassificationSchemeItemRetrieverFactory()
            .newRetriever('classificationScheme').retrieve(wsClient, criteria)
        assert cdeReviewClassificationScheme != null
        assert cdeReviewClassificationScheme.classSchemeClassSchemeItemCollection != null
        cdeReviewClassificationScheme.classSchemeClassSchemeItemCollection.item.sort
        {
            a, b ->
            def csi_a = a.classificationSchemeItem
            def csi_b = b.classificationSchemeItem
            def cmp = (csi_a.longName <=> csi_b.longName)
            if(cmp != 0) return cmp
            cmp = (csi_a.publicID <=> csi_b.publicID)
            if(cmp != 0) return cmp
            return (csi_a.version <=> csi_b.version)
        }
        return cdeReviewClassificationScheme
    }

    def retrieveReleasedFormList(boolean hideNonProtocolReleasedForms)
    {
        def resultList
        def formList = retrieveFormList('RELEASED')
		def trialFormList = retrieveFormList('APPRVD FOR TRIAL USE')
		
        if(hideNonProtocolReleasedForms)
        {
            // filter out forms with empty protocolCollection
            resultList = []
            formList?.each
            {
                def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Form')
                criteria.id = it.id
                def wsResult
                synchronized(wsClient)
                {
                    wsResult = wsClient.queryObject('gov.nih.nci.cadsr.domain.Protocol', criteria)
                }
                if(wsResult.queryObjectReturn?.size > 0)
                {
                    resultList.add(it)
                }
            }
        }
        else
        {
            resultList = formList
        }

        return resultList + trialFormList
    }

    def retrieveDraftNewFormList()
    {
        return retrieveFormList('DRAFT NEW')
    }

    def retrieveFormList(status)
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Form')
        criteria.workflowStatusName = status
        criteria.context = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Context')
        criteria.context.name = 'NHLBI'
        def wsResult
        synchronized(wsClient)
        {
            wsResult = wsClient.queryObject('gov.nih.nci.cadsr.domain.Form', criteria)
        }
        def shallowFormCollection = wsResult.queryObjectReturn
        assert shallowFormCollection != null
        shallowFormCollection.sort
        {
            a, b ->
            def cmp = (a.longName <=> b.longName)
            if(cmp != 0) return cmp
            cmp = (a.publicID <=> b.publicID)
            if(cmp != 0) return cmp
            return (a.version <=> b.version)
        }
        return shallowFormCollection
    }

    def retrieveFormQuestionCount(String id)
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Module')
        criteria.form = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Form')
        criteria.form.id = id
        synchronized(wsClient)
        {
			return 1000
/*            return wsClient.getTotalNumberOfRecords('gov.nih.nci.cadsr.domain.Question', criteria)*/
        }
    }

    def retrieveForm(String id, ProgressMonitor progressMonitor, Boolean expandValueDomain)
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.Form')
        criteria.id = id
        def retriever = new RetrieverFactory(progressMonitor,
            'gov.nih.nci.cadsr.domain.jaxb.Question',
            expandValueDomain).newRetriever('form')
        return retriever.retrieve(wsClient, criteria)
    }

    def retrieveForm(String id, ProgressMonitor progressMonitor)
    {
        return retrieveForm(id, progressMonitor, false)
    }

    def retrieveCscsiItemCount(String id)
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.ClassSchemeClassSchemeItem')
        criteria.id = id
        synchronized(wsClient)
        {
			return 1000
/*            return wsClient.getTotalNumberOfRecords('gov.nih.nci.cadsr.domain.AdministeredComponentClassSchemeItem', criteria) */
        }
    }

    def retrieveClassSchemeClassSchemeItem(String id, ProgressMonitor progressMonitor)
    {
        def criteria = wsClient.create('gov.nih.nci.cadsr.domain.jaxb.ClassSchemeClassSchemeItem')
        criteria.id = id
        def retriever = new CurationRetrieverFactory(progressMonitor,
            'gov.nih.nci.cadsr.domain.jaxb.AdministeredComponentClassSchemeItem')
            .newRetriever('classSchemeClassSchemeItem')
        return retriever.retrieve(wsClient, criteria)
    }

    def marshal(Object obj, String xmlElementName, File outputFile)
    {
        JAXBContext jc = JAXBContext.newInstance(
            'gov.nih.nci.cadsr.domain.jaxb' + ':' +
            'gov.nih.nci.cadsr.umlproject.domain.jaxb' + ':' +
            'gov.nih.nci.cadsrapi.cadsrapi41.services.cadsrapi41service.jaxb')
        Marshaller m = jc.createMarshaller()
        m.setProperty('com.sun.xml.bind.namespacePrefixMapper', new NamespacePrefixMapper());
        m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
        def el = new JAXBElement(
            new QName('urn:domain.cadsr.nci.nih.gov', xmlElementName), obj.class, obj)
        m.marshal(el, outputFile)
    }
}